import React, { useEffect, useRef, useState } from "react";

import "./searchbar.css";
import { NodeData } from "../graph";

interface SearchBarProps {
    minified: boolean;
    nodeSet: NodeData[];
    onSearch?: (node: NodeData) => void;
}

function SearchBar({ minified, nodeSet, onSearch }: SearchBarProps) {
    const [isMinified, setMinified] = useState(minified);
    const [width, setWidth] = useState<number>(minified ? 0 : 100);
    const [inputText, setInputText] = useState("");
    const inputField = useRef<HTMLInputElement>();

    const toggleMinified = () => {
        setMinified((prev) => !prev);
    };

    useEffect(() => {
        if (isMinified) {
            setWidth(0);
            setInputText("");
            inputField.current.value = "";
        } else {
            setWidth(100);
            inputField.current.focus();
        }
    }, [isMinified]);

    const handleInput = (e: React.ChangeEvent<HTMLInputElement>) => {
        setInputText(e.target.value.toLowerCase());
    };

    const labels = nodeSet.map((node) => node.name);
    const nodeLoopup = new Map<string, NodeData>();
    labels.forEach((label, i) => nodeLoopup.set(label, nodeSet[i]));

    const filtered = labels
        .filter((el) => {
            if (inputText !== "") {
                return el.toLowerCase().includes(inputText);
            }
        })
        .slice(0, 3);

    const handleNodeClick = (node: string) => {
        if (onSearch !== undefined) {
            onSearch(nodeLoopup.get(node));
        }
    };

    return (
        <div
            className={"serach-area"}
            style={{ backgroundColor: isMinified ? "" : "white" }}
        >
            <div className={"searchbar"}>
                <div className={"searchbar-icon"} onClick={toggleMinified}>
                    &#128269;
                </div>
                <div
                    className={"searchbar-text"}
                    style={{ width: width + "%" }}
                >
                    <form action={"#"}>
                        <input
                            ref={inputField}
                            className={"searchbar-input"}
                            type={"search"}
                            placeholder={"Knoten suchen..."}
                            onInput={handleInput}
                            onBlur={toggleMinified}
                        ></input>
                    </form>
                </div>
            </div>
            <div
                className={"searchbar-results"}
                style={{
                    height: inputText.length > 0 ? filtered.length * 30 : 0,
                    marginBottom: filtered.length > 0 ? 10 : 0,
                }}
            >
                <ul>
                    {filtered.map((el) => (
                        <li
                            key={el}
                            className={"searchbar-result"}
                            onMouseDown={() => handleNodeClick(el)}
                        >
                            {el}
                        </li>
                    ))}
                </ul>
            </div>
        </div>
    );
}

export default SearchBar;
